local buttonJustPressed = playdate.buttonJustPressed

---@type ButtonState
local buttonState = {}

buttonInputSystem = filteredSystem("buttonInput", { canReceiveButtons = T.marker }, function(e, _, system)
    e.buttonState = buttonState
    system.world:addEntity(e)
end)

function buttonInputSystem:preProcess()
    if #self.entities == 0 then
        return
    end
    buttonState.upJustPressed = buttonJustPressed(playdate.kButtonUp)
    buttonState.downJustPressed = buttonJustPressed(playdate.kButtonDown)
    buttonState.rightJustPressed = buttonJustPressed(playdate.kButtonRight)
    buttonState.leftJustPressed = buttonJustPressed(playdate.kButtonLeft)
    buttonState.aJustPressed = buttonJustPressed(playdate.kButtonA)
    buttonState.bJustPressed = buttonJustPressed(playdate.kButtonB)

    buttonState.receivedInputThisFrame = buttonState.upJustPressed
        or buttonState.downJustPressed
        or buttonState.rightJustPressed
        or buttonState.leftJustPressed
        or buttonState.aJustPressed
        or buttonState.bJustPressed
end

local crankState = {}

crankInputSystem = filteredSystem(
    "crankInput",
    tiny.requireAny("canReceiveCrank", "launchedByCrank"),
    function(e, _, system)
        e.crankState = crankState
        system.world:addEntity(e)
    end
)

crankHistory = filteredSystem("crankHistory", { crankChange = T.number, msOccurred = T.number })

function crankInputSystem:preProcess()
    if #self.entities == 0 then
        return
    end
    local currentMs = playdate.getCurrentTimeMilliseconds()
    local crankChange = playdate.getCrankChange()

    self.world:addEntity({
        crankChange = crankChange,
        msOccurred = currentMs,
    })

    local changeInLastHalfSecond = 0
    for _, v in pairs(crankHistory.entities) do
        if currentMs - v.msOccurred > 500 then
            self.world:removeEntity(v)
        else
            changeInLastHalfSecond = changeInLastHalfSecond + v.crankChange
        end
    end

    crankState.crankChange = crankChange
    crankState.changeInLastHalfSecond = changeInLastHalfSecond
end
